module CustomerServiceIm
  class Message < ApplicationRecord
    belongs_to :user, polymorphic: true
    belongs_to :conversation
    belongs_to :staff, optional: true

    before_validation :set_default_receiver_type, on: :create
    before_validation :set_user, on: :create

    after_create :broadcast_to_conversation #, if: -> { target_receivable? }
    after_create :notice_conversation_message_created

    after_create_commit :speech_recognize, if: ->{ from_user? && speech_unrecognized? }

    has_one_attached :attachment

    scope :from_user, -> {
      where("receiver_type = 'Staff'")
    }

    scope :from_staff, -> {
      where("receiver_type = 'User'")
    }

    scope :id_less_then, ->( id ){
      where('id < ?', id) if id.present?
    }

    scope :unread, ->{
      where.not(state: "read")
    }

    enum state: {
      pending: "pending",
      sent: "sent",
      received: "received",
      read: "read"
    }, _default: "pending"

    def from_user?
      receiver_type === 'Staff'
    end

    def from_staff?
      receiver_type === 'User'
    end

    def target_receivable?
      (from_user? && conversation.staff&.online?) || (from_staff? && user&.online?)
    end

    def broadcast_to_conversation
      message_data = JSON.parse(ApplicationController.render partial: "customer_service_im/messages/message", locals: { message: self})
      CustomerServiceIm::MessagesChannel.broadcast_to(conversation, message_data)

      if target_receivable?
        sent!
      end
    end

    def read!
      conversation.message_read!(self)
      super
    end

    def attachment_url
      return if !attachment.blob

      Rails.application.routes.url_helpers.rails_blob_url(attachment)
    end

    def speech_unrecognized?
      attachment.blob.present? && content.blank?
    end

    def speech_recognize
      result = TxAsr::SentenceRecognition.from(attachment_url)

      if result.success?
        update( content: result.data["Result"].presence || "[空白语音]" )
        broadcast_to_conversation
      else
        logger = Logger.new Rails.root.join("log/speech_recognition.log")
        logger.error result.errors
        raise CustomerServiceIm::AsrRecognizeError.new(result.errors)
      end
    end

    private

    def set_default_receiver_type
      self.receiver_type ||= "Staff"
    end

    def notice_conversation_message_created
      conversation.message_created!(self)
    end

    def set_user
      self.user ||= conversation.user
    end
  end
end
